/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.OutputStream;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.WindowConstants;

import net.sf.distriblend.share.net.Connector;
import net.sf.distriblend.share.net.event.TransferEvent;
import net.sf.distriblend.share.net.event.TransferListener;
import net.sf.distriblend.share.protocol.FileTransfer;
import net.sf.distriblend.share.protocol.Submission.FramesRequest;
import net.sf.distriblend.share.protocol.Submission.FramesResponse;

/**
 * @author Ian Thompson
 * 
 */
public class DownloadFramesDialog {

	/**
	 * 
	 * @param connector
	 * @param projectName
	 * @param saveFile
	 * @return complete
	 */
	public static boolean downloadFrames(Connector connector,
			String projectName, File saveFile) {
		return new DownloadFramesDialog(connector, projectName, saveFile).worker
				.isComplete();
	}

	private Worker worker = null;

	private JDialog dialog = null; // @jve:decl-index=0:visual-constraint="54,34"

	private JPanel contentPane = null;

	private JPanel mainPanel = null;

	private JScrollPane logScrollPane = null;

	private JTextArea logTextArea = null;

	private JProgressBar uploadProgressBar = null;

	private JPanel buttonsPanel = null;

	private JButton doneButton = null;

	private JButton cancelButton = null;

	private JTextPane submitInfoPane = null;

	/**
	 * 
	 * @param connector
	 * @param projectName
	 * @param saveFile
	 */
	private DownloadFramesDialog(Connector connector, String projectName,
			File saveFile) {
		this.worker = new Worker(connector, projectName, saveFile);
		this.worker.start();
		getDialog().setVisible(true);
	}

	/**
	 * This method initializes submitInfoPane
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getSubmitInfoPane() {
		if (submitInfoPane == null) {
			submitInfoPane = new JTextPane();
			submitInfoPane.setEnabled(false);
			submitInfoPane.setDisabledTextColor(getSubmitInfoPane()
					.getForeground());
			submitInfoPane
					.setText("Frames rendered by your project will be downloaded as a zipped archive. If you wish to cancel the download, click Cancel at any time.");
			submitInfoPane.setOpaque(false);
			submitInfoPane.setEditable(false);
		}
		return submitInfoPane;
	}

	/**
	 * This method initializes dialog
	 * 
	 * @return javax.swing.JDialog
	 */
	private JDialog getDialog() {
		if (dialog == null) {
			dialog = new JDialog();
			dialog.setModal(true);
			dialog.setSize(new Dimension(383, 299));
			dialog.setTitle("Download Frames");
			dialog
					.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			dialog.setContentPane(getContentPane());
			dialog.setIconImage(Icons.SUBMIT_PROJECT_DIALOG_ICON.getImage());
		}
		return dialog;
	}

	/**
	 * This method initializes contentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getContentPane() {
		if (contentPane == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints3.gridy = 1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.ipadx = 0;
			gridBagConstraints1.ipady = 0;
			gridBagConstraints1.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridy = 0;
			contentPane = new JPanel();
			contentPane.setLayout(new GridBagLayout());
			contentPane.add(getMainPanel(), gridBagConstraints1);
			contentPane.add(getButtonsPanel(), gridBagConstraints3);
		}
		return contentPane;
	}

	/**
	 * This method initializes mainPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
			gridBagConstraints41.fill = GridBagConstraints.BOTH;
			gridBagConstraints41.gridy = 0;
			gridBagConstraints41.weightx = 1.0;
			gridBagConstraints41.weighty = 0.0;
			gridBagConstraints41.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints41.gridx = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints2.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints2.gridy = 3;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.gridy = 1;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints.gridx = 0;
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.setBorder(BorderFactory
					.createTitledBorder("Image Archive Transfer"));
			mainPanel.add(getLogScrollPane(), gridBagConstraints);
			mainPanel.add(getUploadProgressBar(), gridBagConstraints2);
			mainPanel.add(getSubmitInfoPane(), gridBagConstraints41);
		}
		return mainPanel;
	}

	/**
	 * This method initializes logScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getLogScrollPane() {
		if (logScrollPane == null) {
			logScrollPane = new JScrollPane();
			logScrollPane.setViewportView(getLogTextArea());
		}
		return logScrollPane;
	}

	/**
	 * This method initializes logTextArea
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextArea getLogTextArea() {
		if (logTextArea == null) {
			logTextArea = new JTextArea();
			logTextArea.setEditable(false);
			logTextArea.setFont(getSubmitInfoPane().getFont());
		}
		return logTextArea;
	}

	/**
	 * This method initializes uploadProgressBar
	 * 
	 * @return javax.swing.JProgressBar
	 */
	private JProgressBar getUploadProgressBar() {
		if (uploadProgressBar == null) {
			uploadProgressBar = new JProgressBar();
			uploadProgressBar.setStringPainted(true);
		}
		return uploadProgressBar;
	}

	/**
	 * This method initializes buttonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 1;
			gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints5.gridy = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints4.weightx = 1.0;
			gridBagConstraints4.anchor = GridBagConstraints.EAST;
			buttonsPanel = new JPanel();
			buttonsPanel.setLayout(new GridBagLayout());
			buttonsPanel.add(getDoneButton(), gridBagConstraints4);
			buttonsPanel.add(getCancelButton(), gridBagConstraints5);
		}
		return buttonsPanel;
	}

	/**
	 * This method initializes doneButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDoneButton() {
		if (doneButton == null) {
			doneButton = new JButton();
			doneButton.setText("Finish");
			doneButton.setEnabled(false);
			doneButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getDialog().dispose();
				}
			});
		}
		return doneButton;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					worker.cancel();
					getDialog().dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	private class Worker extends Thread {

		/**
		 * 
		 */
		private boolean complete = false;

		/**
		 * 
		 */
		private final Connector connector;

		/**
		 * 
		 */
		private final String projectName;

		/**
		 * 
		 */
		private final File saveDir;

		/**
		 * 
		 * @param connector
		 * @param projectName
		 * @param saveDir
		 */
		public Worker(Connector connector, String projectName, File saveDir) {
			if (!saveDir.isDirectory())
				throw new IllegalArgumentException("Not a directory");
			this.connector = connector;
			this.projectName = projectName;
			this.saveDir = saveDir;
		}

		/**
		 * @return the complete
		 */
		public boolean isComplete() {
			return complete;
		}

		/**
		 * 
		 * 
		 */
		public void cancel() {

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {

			final JProgressBar pb = getUploadProgressBar();

			/* Initialize. */
			int port;
			getLogTextArea().setText("Initializing...\n");
			pb.setValue(0);
			if (!connector.isSuccessfullyConnected()) {
				getDialog().dispose();
				appendToLog("Failed.");
				JOptionPane
						.showMessageDialog(
								getDialog(),
								"Connection broken. Try reconnecting before downloading.",
								"Connect", JOptionPane.WARNING_MESSAGE,
								Icons.CONNECTION_DISCONNECTED.getImageIcon());
				return;
			}

			/* Send DownloadFrames.Request. */
			appendToLog("Preparing for download...\n");
			pb.setValue(0);
			final int count;
			try {
				connector.send(new FramesRequest(projectName));

				FramesResponse response = (FramesResponse) connector.receive();
				count = response.getFrameCount();
				if (count == 0) {
					appendToLog("Failed.\n");
					pb.setValue(0);
					JOptionPane.showMessageDialog(getDialog(),
							"No frames available.", "Warning",
							JOptionPane.WARNING_MESSAGE);
					return;
				}
				appendToLog("Done.\n");
				pb.setValue(100);

			} catch (IOException e) {
				appendToLog("Failed.\n");
				pb.setValue(0);
				JOptionPane.showMessageDialog(getDialog(),
						"Error in communication. Try reconnecting.", "Warning",
						JOptionPane.WARNING_MESSAGE);
				return;
			}

			/* Download files. */
			appendToLog("Downloading frames...\n");
			pb.setValue(0); // TODO: Proper progress bar.

			boolean success;
			try {
				/* Read directly from socket and save to disk. */
				ObjectInputStream in = connector.getInputStream();

				for (int i = 0; i < count; i++) {
					FileTransfer.Header header = (FileTransfer.Header) connector
							.receive();
					final long size = header.getSize();
					final int ii = i;

					File file = new File(saveDir, header.getPath().getPath());
					file.getParentFile().mkdir();
					OutputStream out = new BufferedOutputStream(
							new FileOutputStream(file));

					TransferListener l = new TransferListener() {
						public void progressChanged(TransferEvent event) {
							long t = event.getTransferred();
							int value = 100;
							if (size > 0)
								value = (int) (100 * (t / size + ii) / count);
							pb.setValue(value);
						}
					};
					FileTransfer.transferData(in, out, size, l);
					out.close();
					pb.setValue(100 * i / count);
				}
				success = true;
			} catch (IOException e) {
				success = false;
				// TODO: Log this
			}

			if (success) {
				pb.setValue(100);
				appendToLog("Download complete.\n");
			} else {
				appendToLog("Failed.\n");
				pb.setValue(0);
				JOptionPane.showMessageDialog(getDialog(),
						"Error downloading.", "Warning",
						JOptionPane.WARNING_MESSAGE);
				return;
			}

			/* Complete. */
			complete = true;
			appendToLog("Finished.\n");
			pb.setValue(100);
			getCancelButton().setEnabled(false);
			getDoneButton().setEnabled(true);

		}

		/**
		 * 
		 * @param s
		 */
		private void appendToLog(final String s) {
			getLogTextArea().append(s);
			getLogScrollPane().getVerticalScrollBar().setValue(
					Integer.MAX_VALUE);
			getLogTextArea().repaint();
		}
	}
}
