/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.util.Enumeration;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.WindowConstants;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreePath;

import net.sf.distriblend.share.FileMap;

/**
 * @author Ian Thompson
 * 
 */
public class FileManager {

	private Frame owner = null;

	private JDialog dialog = null; // @jve:decl-index=0:visual-constraint="0,0"

	private JPanel contentPane = null;

	private JPanel mainPanel = null;

	private JScrollPane remoteTreeScrollPane = null;

	private JTree remoteFileTree = null;

	private ProjectTreeModel remoteTreeModel = null;

	private JScrollPane localTreeScrollPane = null;

	private SystemFileTree localFileTree = null;

	private JScrollPane localListScrollPane = null;

	private SystemFileList localFileList = null;

	private JComboBox drivesComboBox = null;

	private JPanel localButtonsPanel = null;

	private JButton addToProjectButton = null;

	private JPanel remoteButtonsPanel = null;

	private JButton removeButton = null;

	private JPanel dialogButtonsPanel = null;

	private JButton doneButton = null;

	private JTextPane informationPane1 = null;

	private JTextPane informationPane2 = null;

	private JButton mkdirButton = null;

	private JLabel driveLabel = null;

	private JSplitPane splitPane = null;

	private JPanel localPanel = null;

	private JPanel remotePanel = null;

	private JSplitPane localSplitPane = null;

	private JPanel localDirectoriesPanel = null;

	private JPanel localFilesPanel = null;

	private JButton renameButton = null;

	/**
	 * @param owner
	 * @param root
	 */
	public static void showFileManager(Frame owner, IconedNode root) {
		FileManager fm = new FileManager(owner);
		fm.getRemoteTreeModel().setRoot(root);
		fm.getDialog().setVisible(true);
	}

	/**
	 * 
	 * @param owner
	 */
	private FileManager(Frame owner) {
		this.owner = owner;
	}

	/**
	 * This method initializes mainPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.gridy = 0;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.weighty = 1.0;
			gridBagConstraints8.gridwidth = 1;
			gridBagConstraints8.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints8.gridx = 0;
			driveLabel = new JLabel();
			driveLabel.setText("Drive:");
			GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
			gridBagConstraints41.gridx = 0;
			gridBagConstraints41.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints41.gridwidth = 1;
			gridBagConstraints41.insets = new Insets(0, 4, 4, 4);
			gridBagConstraints41.gridy = 1;
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.add(getDialogButtonsPanel(), gridBagConstraints41);
			mainPanel.add(getSplitPane(), gridBagConstraints8);
		}
		return mainPanel;
	}

	/**
	 * This method initializes remoteTreeScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getRemoteTreeScrollPane() {
		if (remoteTreeScrollPane == null) {
			remoteTreeScrollPane = new JScrollPane();
			remoteTreeScrollPane.setViewportView(getRemoteFileTree());
		}
		return remoteTreeScrollPane;
	}

	/**
	 * This method initializes remoteFileTree
	 * 
	 * @return javax.swing.JTree
	 */
	private JTree getRemoteFileTree() {
		if (remoteFileTree == null) {
			DefaultTreeSelectionModel defaultTreeSelectionModel = new DefaultTreeSelectionModel();
			defaultTreeSelectionModel
					.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
			remoteFileTree = new JTree();
			remoteFileTree.setShowsRootHandles(true);
			remoteFileTree.setModel(getRemoteTreeModel());
			remoteFileTree.setCellRenderer(new IconedRenderers.TreeCell());
			remoteFileTree.setSelectionModel(defaultTreeSelectionModel);
			remoteFileTree
					.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
						public void valueChanged(
								javax.swing.event.TreeSelectionEvent e) {
							TreePath path = e.getNewLeadSelectionPath();
							if (path == null) {
								removeButton.setEnabled(false);
								mkdirButton.setEnabled(false);
								renameButton.setEnabled(false);
							} else {
								IconedNode node = (IconedNode) path
										.getLastPathComponent();
								Object o = node.getUserObject();
								if (o instanceof FileMap) {
									FileMap fileMap = (FileMap) o;

									removeButton.setEnabled(true);
									mkdirButton.setEnabled(fileMap
											.isDirectory());
									renameButton.setEnabled(true);
								} else {
									removeButton.setEnabled(false);
									mkdirButton.setEnabled(true);
									renameButton.setEnabled(false);
								}
							}
						}
					});
		}
		return remoteFileTree;
	}

	/**
	 * 
	 * @return ProjectTreeModel
	 */
	private ProjectTreeModel getRemoteTreeModel() {
		if (remoteTreeModel == null) {
			remoteTreeModel = new ProjectTreeModel();
		}
		return remoteTreeModel;
	}

	/**
	 * This method initializes localTreeScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getLocalTreeScrollPane() {
		if (localTreeScrollPane == null) {
			localTreeScrollPane = new JScrollPane();
			localTreeScrollPane.setViewportView(getLocalFileTree());
		}
		return localTreeScrollPane;
	}

	/**
	 * This method initializes localFileTree
	 * 
	 * @return javax.swing.JTree
	 */
	private SystemFileTree getLocalFileTree() {
		if (localFileTree == null) {
			localFileTree = new SystemFileTree(true);
			Object obj = getDrivesComboBox().getSelectedItem();
			if (obj != null)
				localFileTree.setRootPath((File) obj);

			localFileTree.addTreeSelectionListener(new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					TreePath path = localFileTree.getSelectionPath();
					if (path != null) {
						IconedNode node = (IconedNode) path
								.getLastPathComponent();
						getLocalFileTree().populate(node);
						Object o = node.getUserObject();
						if (o instanceof File) {
							File file = (File) o;
							getLocalFileList().listDirectory(file);
							localFileTree.scrollPathToVisible(path);
						}
					}
				}
			});
		}
		return localFileTree;
	}

	/**
	 * This method initializes localListScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getLocalListScrollPane() {
		if (localListScrollPane == null) {
			localListScrollPane = new JScrollPane();
			localListScrollPane.setViewportView(getLocalFileList());
		}
		return localListScrollPane;
	}

	/**
	 * This method initializes localFileList
	 * 
	 * @return javax.swing.JList
	 */
	private SystemFileList getLocalFileList() {
		if (localFileList == null) {
			localFileList = new SystemFileList();
			localFileList.addMouseListener(new java.awt.event.MouseAdapter() {
				@Override
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if (e.getClickCount() == 2) {
						IconedItem item = (IconedItem) localFileList
								.getSelectedValue();
						Object o = item.getUserObject();
						if (o instanceof File)
							localFileTree.selectChild((File) o);
					}
				}
			});
			localFileList
					.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
						public void valueChanged(
								javax.swing.event.ListSelectionEvent e) {
							if (!e.getValueIsAdjusting())
								addToProjectButton
										.setEnabled(e.getFirstIndex() != -1);
						}
					});
		}
		return localFileList;
	}

	/**
	 * This method initializes drivesComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getDrivesComboBox() {
		if (drivesComboBox == null) {
			DefaultComboBoxModel drivesComboBoxModel = new DefaultComboBoxModel();
			for (File root : File.listRoots())
				drivesComboBoxModel.addElement(root);
			drivesComboBox = new JComboBox(drivesComboBoxModel);
			drivesComboBox.setRenderer(new IconedRenderers.SingleIconListCell(
					Icons.DRIVE_HARDDISK.getImageIcon()));
			drivesComboBox
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							localFileTree.setRootPath((File) drivesComboBox
									.getSelectedItem());
							localFileTree.setSelectionRow(0);
						}
					});
		}
		return drivesComboBox;
	}

	/**
	 * This method initializes localButtonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getLocalButtonsPanel() {
		if (localButtonsPanel == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints5.fill = GridBagConstraints.NONE;
			gridBagConstraints5.weightx = 1.0;
			gridBagConstraints5.anchor = GridBagConstraints.EAST;
			localButtonsPanel = new JPanel();
			localButtonsPanel.setLayout(new GridBagLayout());
			localButtonsPanel.add(getAddToProjectButton(), gridBagConstraints5);
		}
		return localButtonsPanel;
	}

	/**
	 * This method initializes addToProjectButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddToProjectButton() {
		if (addToProjectButton == null) {
			addToProjectButton = new JButton();
			addToProjectButton.setText("Add to project");
			addToProjectButton.setEnabled(false);
			addToProjectButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							IconedNode node = (IconedNode) getRemoteFileTree()
									.getLastSelectedPathComponent();
							if (node == null)
								node = getRemoteTreeModel().getRoot();

							/* If this node is a file its parent is a directory. */
							Object o = node.getUserObject();
							if (o instanceof FileMap
									&& !((FileMap) o).isDirectory())
								node = (IconedNode) node.getParent();

							Object[] items = localFileList.getSelectedValues();

							/* Iterate over selected items and add them. */
							for (Object it : items) {
								IconedItem item = (IconedItem) it;
								o = item.getUserObject();
								if (o instanceof File) {
									File file = (File) o;
									getRemoteTreeModel().addToNode(node, file,
											file.getName(), file.isDirectory());
								}
							}
						}
					});
		}
		return addToProjectButton;
	}

	/**
	 * This method initializes remoteButtonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getRemoteButtonsPanel() {
		if (remoteButtonsPanel == null) {
			GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
			gridBagConstraints16.gridx = 1;
			gridBagConstraints16.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints16.anchor = GridBagConstraints.EAST;
			gridBagConstraints16.gridwidth = 1;
			gridBagConstraints16.weightx = 0.0;
			gridBagConstraints16.gridy = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.weightx = 0.0;
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints.gridwidth = 2;
			gridBagConstraints.gridy = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.anchor = GridBagConstraints.EAST;
			gridBagConstraints6.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.gridy = 0;
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.fill = GridBagConstraints.NONE;
			remoteButtonsPanel = new JPanel();
			remoteButtonsPanel.setLayout(new GridBagLayout());
			remoteButtonsPanel.add(getRemoveButton(), gridBagConstraints6);
			remoteButtonsPanel.add(getMkdirButton(), gridBagConstraints);
			remoteButtonsPanel.add(getRenameButton(), gridBagConstraints16);
		}
		return remoteButtonsPanel;
	}

	/**
	 * This method initializes removeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton();
			removeButton.setText("Remove");
			removeButton.setEnabled(false);
			removeButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					int j = JOptionPane
							.showConfirmDialog(
									getDialog(),
									"Are you sure you wish to remove this file or directory from the project?",
									"Remove Item",
									JOptionPane.YES_NO_CANCEL_OPTION, 0,
									Icons.FILE_MANAGER_DIALOG_ICON
											.getImageIcon());

					if (j == JOptionPane.YES_OPTION) {
						IconedNode node = (IconedNode) getRemoteFileTree()
								.getLastSelectedPathComponent();
						if (node != null)
							getRemoteTreeModel().removeFromParent(node);
					}
				}
			});
		}
		return removeButton;
	}

	/**
	 * This method initializes dialogButtonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getDialogButtonsPanel() {
		if (dialogButtonsPanel == null) {
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints7.anchor = GridBagConstraints.EAST;
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.weightx = 1.0;
			dialogButtonsPanel = new JPanel();
			dialogButtonsPanel.setLayout(new GridBagLayout());
			dialogButtonsPanel.add(getDoneButton(), gridBagConstraints7);
		}
		return dialogButtonsPanel;
	}

	/**
	 * This method initializes doneButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDoneButton() {
		if (doneButton == null) {
			doneButton = new JButton();
			doneButton.setText("Done");
			doneButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					dialog.dispose();
				}
			});
		}
		return doneButton;
	}

	/**
	 * This method initializes informationPane1
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getInformationPane1() {
		if (informationPane1 == null) {
			informationPane1 = new JTextPane();
			informationPane1.setOpaque(false);
			informationPane1.setEditable(false);
			informationPane1.setEnabled(false);
			informationPane1.setDisabledTextColor(informationPane1
					.getForeground());
			informationPane1
					.setText("Browse and select files on this side on your local computer. You may choose to add both files and whole directories. However, when adding a directory only the files are added, its sub-directories are not added automatically.");
		}
		return informationPane1;
	}

	/**
	 * This method initializes informationPane2
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getInformationPane2() {
		if (informationPane2 == null) {
			informationPane2 = new JTextPane();
			informationPane2.setOpaque(false);
			informationPane2.setEditable(false);
			informationPane2.setEnabled(false);
			informationPane2.setDisabledTextColor(informationPane2
					.getForeground());
			informationPane2
					.setText("These are your project files in the remote project directory. Add or remove files as necessary. You can also create and organise your files into directories. Remember to make sure your .blend files use only relative paths or pack them.");
		}
		return informationPane2;
	}

	/**
	 * This method initializes mkdirButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getMkdirButton() {
		if (mkdirButton == null) {
			mkdirButton = new JButton();
			mkdirButton.setText("New directory...");
			mkdirButton.setEnabled(false);
			mkdirButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String s = JOptionPane
							.showInputDialog("Enter directory name:");
					if (s != null && s.length() > 0) {
						IconedNode node = (IconedNode) getRemoteFileTree()
								.getLastSelectedPathComponent();
						getRemoteTreeModel().addToNode(node, null, s, true);
					}
				}
			});
		}
		return mkdirButton;
	}

	/**
	 * This method initializes splitPane
	 * 
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getSplitPane() {
		if (splitPane == null) {
			splitPane = new JSplitPane();
			splitPane.setDividerLocation(400);
			splitPane.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
			splitPane.setLeftComponent(getLocalPanel());
			splitPane.setRightComponent(getRemotePanel());
		}
		return splitPane;
	}

	/**
	 * This method initializes localPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getLocalPanel() {
		if (localPanel == null) {
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.fill = GridBagConstraints.BOTH;
			gridBagConstraints17.gridy = 2;
			gridBagConstraints17.weightx = 1.0;
			gridBagConstraints17.weighty = 1.0;
			gridBagConstraints17.insets = new Insets(0, 0, 0, 0);
			gridBagConstraints17.gridx = 0;
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints9.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints9.gridwidth = 1;
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.gridy = 0;
			gridBagConstraints9.weightx = 1.0;
			gridBagConstraints9.weighty = 0.0;
			gridBagConstraints9.fill = GridBagConstraints.HORIZONTAL;
			localPanel = new JPanel();
			localPanel.setLayout(new GridBagLayout());
			localPanel.setBorder(BorderFactory
					.createTitledBorder("Local files"));
			localPanel.add(getInformationPane1(), gridBagConstraints9);
			localPanel.add(getLocalSplitPane(), gridBagConstraints17);
		}
		return localPanel;
	}

	/**
	 * This method initializes remotePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getRemotePanel() {
		if (remotePanel == null) {
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.gridy = 5;
			gridBagConstraints13.anchor = GridBagConstraints.CENTER;
			gridBagConstraints13.weightx = 1.0;
			gridBagConstraints13.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridheight = 1;
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints12.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints12.gridheight = 1;
			gridBagConstraints12.gridx = 0;
			gridBagConstraints12.gridy = 0;
			gridBagConstraints12.weightx = 1.0;
			gridBagConstraints12.weighty = 0.0;
			gridBagConstraints12.fill = GridBagConstraints.HORIZONTAL;
			remotePanel = new JPanel();
			remotePanel.setLayout(new GridBagLayout());
			remotePanel.setBorder(BorderFactory
					.createTitledBorder("Project files"));
			remotePanel.add(getInformationPane2(), gridBagConstraints12);
			remotePanel.add(getRemoteTreeScrollPane(), gridBagConstraints1);
			remotePanel.add(getRemoteButtonsPanel(), gridBagConstraints13);
		}
		return remotePanel;
	}

	/**
	 * This method initializes localSplitPane
	 * 
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getLocalSplitPane() {
		if (localSplitPane == null) {
			localSplitPane = new JSplitPane();
			localSplitPane.setDividerLocation(200);
			localSplitPane.setBorder(BorderFactory
					.createEmptyBorder(0, 0, 0, 0));
			localSplitPane.setLeftComponent(getLocalDirectoriesPanel());
			localSplitPane.setRightComponent(getLocalFilesPanel());
		}
		return localSplitPane;
	}

	/**
	 * This method initializes localDirectoriesPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getLocalDirectoriesPanel() {
		if (localDirectoriesPanel == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.fill = GridBagConstraints.BOTH;
			gridBagConstraints2.gridwidth = 2;
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			gridBagConstraints2.weightx = 1.0;
			gridBagConstraints2.weighty = 1.0;
			gridBagConstraints2.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints10.gridwidth = 1;
			gridBagConstraints10.gridx = 1;
			gridBagConstraints10.gridy = 0;
			gridBagConstraints10.weightx = 1.0;
			gridBagConstraints10.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.anchor = GridBagConstraints.CENTER;
			gridBagConstraints4.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints4.gridx = -1;
			gridBagConstraints4.gridy = -1;
			gridBagConstraints4.fill = GridBagConstraints.HORIZONTAL;
			localDirectoriesPanel = new JPanel();
			localDirectoriesPanel.setLayout(new GridBagLayout());
			localDirectoriesPanel.add(driveLabel, gridBagConstraints4);
			localDirectoriesPanel
					.add(getDrivesComboBox(), gridBagConstraints10);
			localDirectoriesPanel.add(getLocalTreeScrollPane(),
					gridBagConstraints2);
		}
		return localDirectoriesPanel;
	}

	/**
	 * This method initializes localFilesPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getLocalFilesPanel() {
		if (localFilesPanel == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints11.gridwidth = 1;
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.gridy = 2;
			gridBagConstraints11.weightx = 1.0;
			gridBagConstraints11.insets = new Insets(4, 4, 4, 4);
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.gridheight = 2;
			gridBagConstraints3.gridwidth = 1;
			gridBagConstraints3.gridx = -1;
			gridBagConstraints3.gridy = -1;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.weighty = 1.0;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			localFilesPanel = new JPanel();
			localFilesPanel.setLayout(new GridBagLayout());
			localFilesPanel.add(getLocalListScrollPane(), gridBagConstraints3);
			localFilesPanel.add(getLocalButtonsPanel(), gridBagConstraints11);
		}
		return localFilesPanel;
	}

	/**
	 * This method initializes renameButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getRenameButton() {
		if (renameButton == null) {
			renameButton = new JButton();
			renameButton.setText("Rename...");
			renameButton.setEnabled(false);
			renameButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					TreePath treePath = remoteFileTree.getSelectionPath();
					IconedNode node = (IconedNode) treePath
							.getLastPathComponent();

					if (node.getChildCount() == 0) {
						Object o = node.getUserObject();
						if (o instanceof FileMap) {
							FileMap fileMap = (FileMap) o;
							String name = JOptionPane.showInputDialog(
									getDialog(), "Enter new file name:",
									fileMap.getDestinationPath().getName());

							/*
							 * Check nothing exists by the same remoteName (case
							 * insensitive).
							 */
							Enumeration<IconedNode> siblings = node.getParent()
									.children();
							while (siblings.hasMoreElements()) {
								IconedNode sibling = siblings.nextElement();
								o = sibling.getUserObject();
								if (o instanceof FileMap
										&& ((FileMap) o).getDestinationPath()
												.getName().equalsIgnoreCase(
														name)) {
									JOptionPane
											.showMessageDialog(
													getDialog(),
													"A file or directory already exists by that name.",
													"Rename",
													JOptionPane.INFORMATION_MESSAGE,
													Icons.FILE_MANAGER_DIALOG_ICON
															.getImageIcon());
									return;
								}
							}

							fileMap.renameDestination(name);
							getRemoteTreeModel().valueForPathChanged(treePath,
									node);
						}
					} else {
						JOptionPane.showMessageDialog(getDialog(),
								"Can only rename files and empty directories.",
								"Information", JOptionPane.INFORMATION_MESSAGE);
						return;
					}
				}
			});
		}
		return renameButton;
	}

	/**
	 * This method initializes dialog
	 * 
	 * @return javax.swing.JDialog
	 */
	private JDialog getDialog() {
		if (dialog == null) {
			dialog = new JDialog();
			dialog.setSize(new Dimension(694, 465));
			dialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			dialog.setTitle("File Manager");
			dialog.setLocationRelativeTo(owner);
			dialog.setModal(true);
			dialog.setIconImage(Icons.FILE_MANAGER_DIALOG_ICON.getImage());
			dialog.setContentPane(getContentPane());
			dialog.getRootPane().setDefaultButton(doneButton);
		}
		return dialog;
	}

	/**
	 * This method initializes contentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getContentPane() {
		if (contentPane == null) {
			BorderLayout borderLayout = new BorderLayout();
			borderLayout.setVgap(0);
			contentPane = new JPanel();
			contentPane.setLayout(borderLayout);
			contentPane.add(getMainPanel(), BorderLayout.CENTER);
		}
		return contentPane;
	}

}
