/*
 * Distriblend: A distributed rendering system for Blender Copyright (C) 2007:
 * Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.Component;
import java.io.File;

import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JList;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

/**
 * @author Ian Thompson
 * 
 */
public class IconedRenderers {

	/**
	 * @author Ian Thompson
	 * 
	 */
	public static class TreeCell extends DefaultTreeCellRenderer {

		/**
		 * 
		 */
		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value,
				boolean sel, boolean expanded, boolean leaf, int row,
				boolean hasFocus) {

			Component c = super.getTreeCellRendererComponent(tree, value, sel,
					expanded, leaf, row, hasFocus);
			if (value instanceof IconedNode) {
				IconedNode node = (IconedNode) value;

				/* Certain types get special treatment */
				Object o = node.getUserObject();
				if (o instanceof File)
					setText(fileToString((File) o));
				else
					setText(node.getUserObject().toString());
				setIcon(node.getIcon());
				return this;
			} else {
				setText(value.toString());
				setIcon(null);
			}
			return c;
		}
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	public static class ListCell extends DefaultListCellRenderer {

		/**
		 * 
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {

			Component c = super.getListCellRendererComponent(list, value,
					index, isSelected, cellHasFocus);
			if (value instanceof IconedItem) {
				IconedItem item = (IconedItem) value;

				/* Certain types get special treatment */
				Object o = item.getUserObject();
				if (o instanceof File)
					setText(fileToString((File) o));
				else
					setText(o.toString());
				setIcon(item.getIcon());
			}
			return c;
		}
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	public static class SingleIconListCell extends DefaultListCellRenderer {

		private final Icon icon;

		/**
		 * @param icon
		 * 
		 */
		public SingleIconListCell(Icon icon) {
			super();
			this.icon = icon;
		}

		/**
		 * 
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			Component c = super.getListCellRendererComponent(list, value,
					index, isSelected, cellHasFocus);
			setIcon(icon);
			return c;
		}

	}

	/**
	 * Private constructor.
	 */
	private IconedRenderers() {
	}

	/**
	 * Returns the name of the file, directory or drive represented by the
	 * abstract pathname specified.
	 * 
	 * @param file
	 *            The abstract pathname to represent.
	 * @return The name of the file, directory or drive.
	 */
	private static String fileToString(File file) {
		String name = file.getName();
		if (name.equals(""))
			return file.getPath();
		else
			return name;
	}
}
