/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import net.sf.distriblend.share.protocol.Submission.MonitorItem;

/**
 * @author Ian Thompson
 * 
 */
public class MonitorListItem extends JPanel {

	/** */
	public static final String PAUSE_ACTION_COMMAND = "pause";

	/** */
	public static final String CANCEL_ACTION_COMMAND = "cancel";

	/** */
	public static final String DOWNLOAD_ACTION_COMMAND = "download";

	/** */
	public static final String DELETE_ACTION_COMMAND = "delete";

	private JLabel projectLabel = null;

	private JButton pauseResumeButton = null;

	private JButton cancelButton = null;

	private JProgressBar progressBar = null;

	private JButton downloadImagesButton = null;

	private JButton deleteButton = null;

	/**
	 * 
	 */
	public MonitorListItem() {
		initialize();
	}

	/**
	 * 
	 * @param l
	 */
	public void addActionListener(ActionListener l) {
		getPauseResumeButton().addActionListener(l);
		getCancelButton().addActionListener(l);
		getDownloadImagesButton().addActionListener(l);
		getDeleteButton().addActionListener(l);
	}

	/**
	 * 
	 * @param l
	 */
	public void removeActionListener(ActionListener l) {
		getPauseResumeButton().removeActionListener(l);
		getCancelButton().removeActionListener(l);
		getDownloadImagesButton().removeActionListener(l);
		getDeleteButton().removeActionListener(l);
	}

	/**
	 * 
	 * @param mi
	 */
	public void setMonitorData(MonitorItem mi) {
		getProjectLabel().setText(mi.projectName);
		getProgressBar().setString(
				mi.framesRendered + " of " + mi.framesToRender
						+ " frames rendered.");

		if (mi.framesToRender != 0) {
			if (mi.framesRendered == mi.framesToRender) {

				getPauseResumeButton().setEnabled(false);
				getCancelButton().setEnabled(false);
				getDownloadImagesButton().setEnabled(true);
				getDeleteButton().setEnabled(true);

				getProjectLabel().setIcon(Icons.PROJECT.getImageIcon());
				getDownloadImagesButton().setEnabled(true);
				getDeleteButton().setEnabled(true);
			} else {

				getPauseResumeButton().setEnabled(true);
				getCancelButton().setEnabled(true);
				getDownloadImagesButton().setEnabled(false);
				getDeleteButton().setEnabled(false);

				getProjectLabel().setIcon(Icons.PROJECT_RUN.getImageIcon());
			}
			int percent = (100 * mi.framesRendered) / mi.framesToRender;
			getProgressBar().setValue(percent);
		} else {
			getPauseResumeButton().setEnabled(false);
			getCancelButton().setEnabled(false);
			getDownloadImagesButton().setEnabled(false);
			getDeleteButton().setEnabled(true);

			getProjectLabel().setIcon(Icons.PROJECT.getImageIcon());
			getProgressBar().setValue(0);
		}
	}

	/**
	 * 
	 * @return Project name
	 */
	public String getProjectName() {
		return projectLabel.getText();
	}

	/**
	 * This method initializes panel
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
		gridBagConstraints13.gridx = 4;
		gridBagConstraints13.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints13.gridwidth = 1;
		gridBagConstraints13.anchor = GridBagConstraints.EAST;
		gridBagConstraints13.gridy = 0;
		GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
		gridBagConstraints12.gridx = 3;
		gridBagConstraints12.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints12.gridwidth = 1;
		gridBagConstraints12.anchor = GridBagConstraints.EAST;
		gridBagConstraints12.gridy = 0;
		GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
		gridBagConstraints6.gridx = 0;
		gridBagConstraints6.gridwidth = 5;
		gridBagConstraints6.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints6.weightx = 1.0;
		gridBagConstraints6.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints6.weighty = 0.0;
		gridBagConstraints6.gridy = 1;
		GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
		gridBagConstraints5.gridx = 2;
		gridBagConstraints5.anchor = GridBagConstraints.EAST;
		gridBagConstraints5.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints5.gridwidth = 1;
		gridBagConstraints5.weightx = 0.0;
		gridBagConstraints5.weighty = 0.0;
		gridBagConstraints5.fill = GridBagConstraints.NONE;
		gridBagConstraints5.gridy = 0;
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints1.anchor = GridBagConstraints.WEST;
		gridBagConstraints1.weightx = 0.0;
		gridBagConstraints1.fill = GridBagConstraints.NONE;
		gridBagConstraints1.weighty = 0.0;
		gridBagConstraints1.gridy = 0;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 0.0;
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.gridwidth = 1;
		gridBagConstraints.gridy = 0;

		this.setLayout(new GridBagLayout());
		this.setSize(new Dimension(379, 66));
		this.setOpaque(true);
		this.setBorder(BorderFactory.createEmptyBorder());
		this.setMinimumSize(new Dimension(200, 48));
		this.add(getProjectLabel(), gridBagConstraints);
		this.add(getPauseResumeButton(), gridBagConstraints1);
		this.add(getCancelButton(), gridBagConstraints5);
		this.add(getProgressBar(), gridBagConstraints6);
		this.add(getDownloadImagesButton(), gridBagConstraints12);
		this.add(getDeleteButton(), gridBagConstraints13);
	}

	private JLabel getProjectLabel() {
		if (projectLabel == null) {
			projectLabel = new JLabel();
			projectLabel.setText("");
			projectLabel.setIcon(Icons.PROJECT.getImageIcon());
		}
		return projectLabel;
	}

	/**
	 * This method initializes pauseResumeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getPauseResumeButton() {
		if (pauseResumeButton == null) {
			pauseResumeButton = new JButton();
			pauseResumeButton.setText("Pause");
			pauseResumeButton.setVisible(false);
			pauseResumeButton.setActionCommand(PAUSE_ACTION_COMMAND);
		}
		return pauseResumeButton;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.setVisible(false);
			cancelButton.setActionCommand(CANCEL_ACTION_COMMAND);
		}
		return cancelButton;
	}

	/**
	 * This method initializes progressBar
	 * 
	 * @return javax.swing.JProgressBar
	 */
	private JProgressBar getProgressBar() {
		if (progressBar == null) {
			progressBar = new JProgressBar();
			progressBar.setStringPainted(true);
		}
		return progressBar;
	}

	/**
	 * This method initializes downloadImagesButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDownloadImagesButton() {
		if (downloadImagesButton == null) {
			downloadImagesButton = new JButton();
			downloadImagesButton.setText("Save images...");
			downloadImagesButton.setActionCommand(DOWNLOAD_ACTION_COMMAND);
			downloadImagesButton.setEnabled(false);
		}
		return downloadImagesButton;
	}

	/**
	 * This method initializes deleteButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDeleteButton() {
		if (deleteButton == null) {
			deleteButton = new JButton();
			deleteButton.setText("Delete");
			deleteButton.setActionCommand(DELETE_ACTION_COMMAND);
			deleteButton.setEnabled(false);
		}
		return deleteButton;
	}
}