/*
 * Distriblend: A distributed rendering system for Blender
 * Copyright (C) 2007: Ian Philip Thompson
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.distributor;

import java.util.TreeMap;


/**
 * @author Ian Thompson
 * 
 */
public class UserList {

	/**
	 * 
	 * @return The single {@link UserList}
	 */
	public static UserList instance() {
		if (instance == null) {
			instance = new UserList();
		}
		return instance;
	}

	/**
	 * Logs the given user into this {@link UserList}. First finding any
	 * previous logged in user by the same name and disconnecting it.
	 * 
	 * @param userName
	 * @param clientHandler
	 */
	public void logUser(String userName, ClientHandler clientHandler) {

		/* Disconnect existing connection. */
		Profile profile = userMap.get(userName);
		if (profile != null) {
			profile.clientHandler.disconnect();
			userMap.remove(userName);
		}

		/* Add the new connection to the list. */
		profile = new Profile(clientHandler);
		userMap.put(userName, profile);
	}

	/**
	 * 
	 */
	private TreeMap<String, Profile> userMap;

	/**
	 * 
	 */
	private static UserList instance = null;

	/**
	 * 
	 * 
	 */
	private UserList() {
		this.userMap = new TreeMap<String, Profile>();
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	private class Profile {

		long connectedAt;

		ClientHandler clientHandler;

		/**
		 * 
		 * @param clientHandler
		 */
		public Profile(ClientHandler clientHandler) {
			this.connectedAt = System.currentTimeMillis();
			this.clientHandler = clientHandler;
		}
	}
}
