/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.node.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.border.TitledBorder;

class WizardStep4 extends WizardStep {

	private JPanel spacerPanel = null;

	private JTextPane informationText = null;

	private JButton browseButton = null;

	private ButtonGroup radioButtonGroup = null;

	private JRadioButton customRadio = null;

	private JRadioButton workingDirectoryRadio = null;

	private JRadioButton userHomeRadio = null;

	private JTextField settingsFileField = null;

	private JFileChooser fileChooser = null;

	/**
	 * This method initializes
	 * 
	 */
	public WizardStep4() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints33 = new GridBagConstraints();
		gridBagConstraints33.fill = GridBagConstraints.BOTH;
		gridBagConstraints33.gridy = 5;
		gridBagConstraints33.weightx = 1.0;
		gridBagConstraints33.insets = new Insets(4, 32, 16, 4);
		gridBagConstraints33.gridx = 0;
		GridBagConstraints gridBagConstraints32 = new GridBagConstraints();
		gridBagConstraints32.gridx = 0;
		gridBagConstraints32.insets = new Insets(0, 16, 8, 16);
		gridBagConstraints32.anchor = GridBagConstraints.WEST;
		gridBagConstraints32.gridwidth = 2;
		gridBagConstraints32.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints32.gridy = 3;
		GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
		gridBagConstraints31.gridx = 0;
		gridBagConstraints31.insets = new Insets(8, 16, 0, 16);
		gridBagConstraints31.anchor = GridBagConstraints.WEST;
		gridBagConstraints31.gridwidth = 2;
		gridBagConstraints31.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints31.gridy = 2;
		GridBagConstraints gridBagConstraints30 = new GridBagConstraints();
		gridBagConstraints30.gridx = 0;
		gridBagConstraints30.insets = new Insets(0, 16, 4, 16);
		gridBagConstraints30.anchor = GridBagConstraints.WEST;
		gridBagConstraints30.gridwidth = 2;
		gridBagConstraints30.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints30.gridy = 4;
		GridBagConstraints gridBagConstraints29 = new GridBagConstraints();
		gridBagConstraints29.gridx = 1;
		gridBagConstraints29.insets = new Insets(4, 4, 16, 16);
		gridBagConstraints29.fill = GridBagConstraints.VERTICAL;
		gridBagConstraints29.gridy = 5;
		GridBagConstraints gridBagConstraints28 = new GridBagConstraints();
		gridBagConstraints28.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints28.gridy = 0;
		gridBagConstraints28.weightx = 1.0;
		gridBagConstraints28.weighty = 0.0;
		gridBagConstraints28.gridwidth = 2;
		gridBagConstraints28.insets = new Insets(4, 4, 4, 4);
		gridBagConstraints28.gridx = 0;
		GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
		gridBagConstraints17.gridx = 0;
		gridBagConstraints17.weightx = 1.0;
		gridBagConstraints17.weighty = 1.0;
		gridBagConstraints17.fill = GridBagConstraints.BOTH;
		gridBagConstraints17.gridwidth = 2;
		gridBagConstraints17.gridy = 6;
		this.setLayout(new GridBagLayout());
		this.setBorder(new TitledBorder(null, "Step 4 of 4",
				TitledBorder.TRAILING, TitledBorder.TOP));
		this.add(getSpacerPanel(), gridBagConstraints17);
		this.add(getInformationText(), gridBagConstraints28);
		this.add(getBrowseButton(), gridBagConstraints29);
		this.add(getCustomRadio(), gridBagConstraints30);
		this.add(getWorkingDirectoryRadio(), gridBagConstraints31);
		this.add(getUserHomeRadio(), gridBagConstraints32);
		this.add(getSettingsFileField(), gridBagConstraints33);
	}

	/**
	 * This method initializes spacerPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getSpacerPanel() {
		if (spacerPanel == null) {
			spacerPanel = new JPanel();
			spacerPanel.setLayout(new GridBagLayout());
		}
		return spacerPanel;
	}

	/**
	 * This method initializes informationText
	 * 
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getInformationText() {
		if (informationText == null) {
			informationText = new JTextPane();
			informationText.setOpaque(false);
			informationText.setDisabledTextColor(getInformationText()
					.getForeground());
			informationText.setEditable(false);
			informationText
					.setText("Select a suitable location to save the node.conf file:");
		}
		return informationText;
	}

	/**
	 * This method initializes blenderCommandBrowseButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getBrowseButton() {
		if (browseButton == null) {
			browseButton = new JButton();
			browseButton.setText("Browse...");
			browseButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String path = getSettingsFileField().getText();
					getFileChooser().setSelectedFile(new File(path));
					int result = getFileChooser().showSaveDialog(
							WizardStep4.this);
					if (result == JFileChooser.APPROVE_OPTION) {
						path = getFileChooser().getSelectedFile()
								.getAbsolutePath();
						getSettingsFileField().setText(path);
					}
				}
			});
		}
		return browseButton;
	}

	/**
	 * This method initializes directoryChooser
	 * 
	 * @return javax.swing.JFileChooser
	 */
	private JFileChooser getFileChooser() {
		if (fileChooser == null) {
			fileChooser = new JFileChooser();
			fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		}
		return fileChooser;
	}

	private ButtonGroup getRadioButtonGroup() {
		if (radioButtonGroup == null) {
			radioButtonGroup = new ButtonGroup();
		}
		return radioButtonGroup;
	}

	/**
	 * This method initializes customRadio
	 * 
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getCustomRadio() {
		if (customRadio == null) {
			customRadio = new JRadioButton();
			customRadio.setText("Choose a directory... (you will need to use "
					+ "the -s option to load this file later)");
			customRadio.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getSettingsFileField().setEnabled(true);
					getBrowseButton().setEnabled(true);
				}
			});
			getRadioButtonGroup().add(customRadio);
		}
		return customRadio;
	}

	/**
	 * This method initializes workingDirectoryRadio
	 * 
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getWorkingDirectoryRadio() {
		if (workingDirectoryRadio == null) {
			workingDirectoryRadio = new JRadioButton();
			workingDirectoryRadio.setText("In the current working directory");
			workingDirectoryRadio.setSelected(true);
			workingDirectoryRadio
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							getSettingsFileField().setText(
									System.getProperty("user.dir")
											+ File.separator + "node.conf");
							getSettingsFileField().setEnabled(false);
							getBrowseButton().setEnabled(false);
						}
					});
			getRadioButtonGroup().add(workingDirectoryRadio);
		}
		return workingDirectoryRadio;
	}

	/**
	 * This method initializes userHomeRadio
	 * 
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getUserHomeRadio() {
		if (userHomeRadio == null) {
			userHomeRadio = new JRadioButton();
			userHomeRadio.setText("In the current user's home directory");
			userHomeRadio
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							getSettingsFileField().setText(
									System.getProperty("user.home")
											+ File.separator + ".distriblend"
											+ File.separator + "node.conf");
							getSettingsFileField().setEnabled(false);
							getBrowseButton().setEnabled(false);
						}
					});
			getRadioButtonGroup().add(userHomeRadio);
		}
		return userHomeRadio;
	}

	/**
	 * This method initializes settingsFileField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getSettingsFileField() {
		if (settingsFileField == null) {
			settingsFileField = new JTextField();
			settingsFileField.setEnabled(false);
			settingsFileField.setText(System.getProperty("user.dir")
					+ File.separator + "node.conf");
		}
		return settingsFileField;
	}

	@Override
	boolean validateForm() {
		File f = new File(getSettingsFileField().getText());
		if (f.exists()) {
			int result = JOptionPane.showConfirmDialog(this,
					"Overwrite existing configuration?", "File Exists",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE);
			if (result != JOptionPane.YES_OPTION)
				return false;
		}

		return true;
	}

	/**
	 * @return the file to which the adjusted settings should be saved.
	 */
	public File getFile() {
		return new File(getSettingsFileField().getText());
	}
}
