/*
 * Distriblend: A distributed rendering system for Blender
 * Copyright (C) 2007: Ian Philip Thompson
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share.event;

/**
 * An event fired when a setting is changed.
 * 
 * @author Ian Thompson
 * 
 */
public class SettingChangedEvent {

	private String key;

	private String oldValue;

	private String newValue;

	/**
	 * Creates a new event.
	 * 
	 * @param key
	 *            the key of the setting that has changed
	 * @param oldValue
	 *            the value of the setting before the change occured
	 * @param newValue
	 *            the value of the setting after the change occured
	 */
	public SettingChangedEvent(String key, String oldValue,
			String newValue) {
		super();
		this.key = key;
		this.oldValue = oldValue;
		this.newValue = newValue;
	}

	/**
	 * Returns the key of the changed setting.
	 * 
	 * @return the key of the changed setting
	 */
	public String getKey() {
		return key;
	}

	/**
	 * Returns the value of the setting after the change occured.
	 * 
	 * @return the new value of the setting
	 */
	public String getNewValue() {
		return newValue;
	}

	/**
	 * Returns the value of the setting before the change occured.
	 * 
	 * @return the old value of the setting
	 */
	public String getOldValue() {
		return oldValue;
	}
}
