/*
 * Distriblend: A distributed rendering system for Blender Copyright (C) 2007:
 * Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share.net;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import net.sf.distriblend.share.HashFunction;
import net.sf.distriblend.share.net.event.ConnectionEvent;
import net.sf.distriblend.share.net.event.ConnectionListener;
import net.sf.distriblend.share.protocol.Authorization;

/**
 * @author Ian Thompson
 * 
 */
public class AuthorizedConnector extends Connector implements
		ConnectionListener {

	/**
	 * 
	 * @param host
	 * @param port
	 * @param timeout
	 */
	public AuthorizedConnector(String host, int port, int timeout) {
		super(host, port, timeout);
		addConnectionListener(this);
	}

	/**
	 * 
	 * @param user
	 * @param password
	 * @return boolean
	 * @see Authorization
	 */
	public boolean authorize(String user, String password) {
		Object o;
		if (isAuthorized())
			return true;
		this.user = user;
		try {
			Authorization.Request request = new Authorization.Request(user);
			send(request);

			o = receive();
			if (!(o instanceof Authorization.Ready)) {
				close();
				return false;
			}
			Authorization.Ready ready = (Authorization.Ready) o;
			String salt = ready.getSalt();

			String hashed = HashFunction.getDefault().hash(password + salt);
			Authorization.Data data = new Authorization.Data(hashed);
			send(data);

			o = receive();
			if (!(o instanceof Authorization.Response)) {
				close();
				return false;
			}
			Authorization.Response response = (Authorization.Response) o;
			if (response.isAccepted()) {
				authorized = true;
				return true;
			} else {
				close();
				return false;
			}
		} catch (IOException e) {
			close();
			return false;
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			close();
			return false;
		}
	}

	/**
	 * 
	 * @return boolean
	 */
	public boolean isAuthorized() {
		return isSuccessfullyConnected() && authorized;
	}

	/**
	 * @return User
	 */
	public String getUser() {
		return user;
	}

	/**
	 * 
	 */
	private boolean authorized = false;

	private String user = null;

	/**
	 * {@inheritDoc}
	 */
	public void connectionEstablished(ConnectionEvent evt) {
	}

	/**
	 * {@inheritDoc}
	 */
	public void connectionLost(ConnectionEvent evt) {
		authorized = false;
	}
}
